'use strict'

const path = require('node:path')
const AutoLoad = require('@fastify/autoload')
const {db} = require("./helpers/db");
const {pitLib} = require("./helpers/pitLib");
const {Token} = require("./model/TokenModel");

// Pass --options via CLI arguments in command to enable these options.
const options = {}

module.exports = async function (fastify, opts) {
  // Place here your custom code!

  // Do not touch the following lines

  // This loads all plugins defined in plugins
  // those should be support plugins that are reused
  // through your application
  fastify.register(AutoLoad, {
    dir: path.join(__dirname, 'plugins'),
    options: Object.assign({}, opts)
  })
  console.log(db)
  await fastify.register(require('@fastify/cors'), {
    origin: '*',
    methods: ['GET', 'PUT', 'POST', 'DELETE'],
    allowedHeaders: ['Content-Type', 'Authorization'],
    credentials: true,
  })
  fastify.register(require('@fastify/jwt'), {secret: 'supersecret'})
  fastify.register(require('@fastify/multipart'), { attachFieldsToBody: 'keyValues',limits: { fileSize: 52428800 },preservePath: true,onFile })
  fastify.register(AutoLoad, {
    dir: path.join(__dirname, 'routes'),
    options: Object.assign({}, opts)
  })
  fastify.setErrorHandler(function (error, request, reply) {
    reply.send(pitLib.sendResponse(0,error.toString(),null))
  })
  fastify.addHook('preHandler', (request, reply, done) => {
    // Modify the request body as needed
    request.body=(request.body===undefined)?{}:request.body
    done();
  });
  fastify.decorate("authenticate", async function(request, reply) {
    try {
      let decodedToken=await request.jwtVerify()
      if (decodedToken) {
        let token=await Token.findOne({token:request.headers.authorization.replaceAll('Bearer ','')})
        if (!token||!token.active) {reply.send(pitLib.sendResponse(false,'Invalid Token please login'))}
        request.headers.user_data=decodedToken
      }
    } catch (err) {
      reply.send(pitLib.sendResponse(0,err.toString()))
    }
  })
}
async function onFile(part) {
  const buff = await part.toBuffer()
  part.value = {file:Buffer.from(buff, 'binary'),name:part['filename']} // set `part.value` to specify the request body value
}

module.exports.options = options
