const {pitLib} = require("../helpers/pitLib");
const {Plan} = require("../model/PlanModel");
const Razorpay = require("razorpay");
const {User} = require("../model/user-model");
const {PlanPurchase} = require("../model/PlanPurchaseModel");
const {PlanMembership} = require("../model/PlanMembership");
const {DietPlan} = require("../model/DietPlan");
const rzp = new Razorpay({ key_id: process.env.RZP_KEY_TEST, key_secret: process.env.RZP_KEY_SECRET_TEST, headers : { 'Content-Type': 'application/json' }})

const create = async (req, res) => {
    r=await pitLib.service.create(Plan,req.body)
    return res.send(pitLib.sendResponse(!r.err,(r.err)?r.err:null,r?.res))
};
const list  = async (req, res) => {
    r=await Plan.find(req.body).lean()
    return res.send(pitLib.sendResponse(true,null,r))
}
const purchase = async (req, res) => {
    p=await Plan.findById(req.body._id)
    currencyToBe=null
    if (req.headers.user_data.country=='661bbed852f0bc15cfd20b97'){
        currencyToBe="inr"
    } else {
        currencyToBe="usd"
    }
    let currentTime = new Date().getTime();

// Calculate the timestamp for 15 minutes in the future (in milliseconds)
    let futureTime = currentTime + (15 * 60 * 1000); // 15 minutes * 60 seconds * 1000 milliseconds

// Convert the future timestamp to seconds
    let expireBy = Math.floor(futureTime / 1000)*60;

    const myHeaders = new Headers();
    myHeaders.append("Content-Type", "application/json");
    myHeaders.append("Authorization", "Basic cnpwX3Rlc3RfR1lMNnBCTnptM0VaT2M6RVBQVWo3dEpiYm56ODVzSDFkNnRRVXRr");

    const raw = JSON.stringify({
        "amount": p.price[currencyToBe]*100,
        "currency": currencyToBe.toUpperCase(),
        "accept_partial": false,
        "expire_by": expireBy,
        "reference_id": pitLib.util.uniquid(),
        "description": p.title+" Plan",
        "customer": {
            "name": req.headers.user_data.name??'',
            "contact": req.headers.user_data.phone??'',
            "email": req.headers.user_data.email??''
        },
        "notify": {
            "sms": true,
            "email": true
        },
        "reminder_enable": true,
        "notes": {
            "_id": p._id,
        },
        "callback_url": process.env.BACK_END+'api/plan/purchase/handle',
        "callback_method": "get"
    });

    const requestOptions = {
        method: "POST",
        headers: myHeaders,
        body: raw,
        redirect: "follow"
    };

    d=await fetch("https://api.razorpay.com/v1/payment_links", requestOptions)
        .then((response) => response.json())
        // .then((result) => res.redirect(result.short_url))
        .then((result) => res.send(pitLib.sendResponse(!result.error,(result.error)?result.error.description:'Action perfomed successfully',(!result.error)?result:null)))
        .catch((error) => res.send(pitLib.sendResponse(0,error.description)));
};
const handlePurchase =async (req,res)=>{
    let p=await rzp.paymentLink.fetch(req.query.razorpay_payment_link_id)
    plan=await Plan.findById(p.notes._id)
    let u=await User.findOne({email:p.customer.email}).lean()
    if (p.status==="paid"){
        entryVals=p
        entryVals.plan_id=p.notes._id
        entryVals.created_by=u._id
        entryVals.payment_link_id=p.id
        let purchase = await PlanPurchase.create(entryVals)
        let membership= await PlanMembership.create({
            plan:p.notes._id,
            purchase:purchase._id,
            expiry:pitLib.date.addDays(new Date(),plan.validity),
            created_by:u._id
        })
    }
    return res.redirect(process.env.FRONT_END)
}
const purchaseList =async (req,res)=>{
    total=parseInt(await PlanPurchase.countDocuments())
    query=[
        {$match: req.body,},
        {$skip: Number(req.query.page-1)*req.query.pagination,},
        {$limit: Number(req.query.pagination),},
        {$lookup:{from: 'users', localField: 'created_by', foreignField: '_id', as: 'created_by'}},
        {$lookup:{from: 'plans', localField: 'plan_id', foreignField: '_id', as: 'plan_id'}},
        {$lookup:{from: 'planmemberships', localField: '_id', foreignField: 'purchase', as: 'membership'}},
    ]

    filtered=await PlanPurchase.aggregate(query)
    r={
        total:total,
        pagination:eval(req.query.pagination),
        page:eval(req.query.page),
        pages:(total/eval(req.query.pagination)).toFixed(),
        filtered:filtered.length,
        data:filtered
    }
    return res.send(pitLib.sendResponse(true,null,r))
}
const membersipList =async (req,res)=>{
    total=parseInt(await PlanMembership.countDocuments())
    query=[
        {$match: req.body,},
        {$skip: Number(req.query.page-1)*req.query.pagination,},
        {$limit: Number(req.query.pagination),},
        {$lookup:{from: 'users', localField: 'created_by', foreignField: '_id', as: 'created_by',pipeline:[{$project:{name:1}}]}},
        {$lookup:{from: 'plans', localField: 'plan', foreignField: '_id', as: 'plan'}},
        {$lookup:{from: 'planpurchases', localField: 'purchase', foreignField: '_id', as: 'purchase'}},
    ]

    filtered=await PlanMembership.aggregate(query)
    r={
        total:total,
        pagination:eval(req.query.pagination),
        page:eval(req.query.page),
        pages:(total/eval(req.query.pagination)).toFixed(),
        filtered:filtered.length,
        data:filtered
    }
    return res.send(pitLib.sendResponse(true,null,r))
}
const dietPlanList =async (req,res)=>{
    total=parseInt(await DietPlan.countDocuments())
    query=[
        {$match: req.body,},
        {$skip: Number(req.query.page-1)*req.query.pagination,},
        {$limit: Number(req.query.pagination),},
        {$lookup:{from: 'users', localField: 'created_by', foreignField: '_id', as: 'created_by',pipeline:[{$project:{name:1}}]}},
        // {$lookup:{from: 'plans', localField: 'plan_id', foreignField: '_id', as: 'plan_id'}},
        {$lookup:{from: 'planmemberships', localField: 'plan_membership', foreignField: '_id', as: 'plan_membership'}},
    ]

    filtered=await DietPlan.aggregate(query)
    r={
        total:total,
        pagination:eval(req.query.pagination),
        page:eval(req.query.page),
        pages:(total/eval(req.query.pagination)).toFixed(),
        filtered:filtered.length,
        data:filtered
    }
    return res.send(pitLib.sendResponse(true,null,r))
}
const dietPlanCreate =async (req,res)=>{
    r=await pitLib.service.create(DietPlan,req.body)
    return res.send(pitLib.sendResponse(!r.err,(r.err)?r.err:null,r?.res))
}
module.exports ={create,list,purchase,handlePurchase,purchaseList,membersipList,dietPlanCreate,dietPlanList}