'use strict'
const UserController=require('../../controller/user-controller')
const UserValidation=require('../../validation/user-validation')
const PlanValidation=require('../../validation/plan-validation')
const PlanController=require('../../controller/plan-controller')
const {GoogleController} = require("../../controller/GoogleController");
const {GoogleValidation} = require("../../validation/GoogleValidation");
const {CountryController} = require("../../controller/CountryController");
const {CountryValidation} = require("../../validation/CountryValidation");
const {PaginationMiddleware} = require("../../middlewares/UtilMiddleware");

module.exports = async function (fastify, opts) {
  fastify.post('/user/create',{preHandler:[UserValidation.create]}, UserController.create)
  fastify.post('/user/update',{preHandler:[UserValidation.update]}, UserController.update)
  fastify.post('/user/login',{preHandler:[UserValidation.login]}, UserController.login)
  fastify.post('/user/list',{preHandler:[fastify.authenticate,PaginationMiddleware.pagination,UserValidation.list]}, UserController.list)

  fastify.post('/plan/create',{preHandler:[fastify.authenticate,PlanValidation.create]}, PlanController.create)
  fastify.post('/plan/list',{preHandler:[PlanValidation.list]}, PlanController.list)
  fastify.post('/plan/purchase',{preHandler:[fastify.authenticate,PlanValidation.purchase]}, PlanController.purchase)
  fastify.post('/plan/purchase/list',{preHandler:[fastify.authenticate,PaginationMiddleware.pagination,PlanValidation.purchaseList]}, PlanController.purchaseList)
  fastify.post('/plan/membership/list',{preHandler:[fastify.authenticate,PaginationMiddleware.pagination,PlanValidation.membersipList]}, PlanController.membersipList)
  fastify.get('/plan/purchase/handle',{preHandler:[/*fastify.authenticatePlanValidation.purchase*/]}, PlanController.handlePurchase)

  fastify.post('/plan/membership/diet-plan/create',{preHandler:[fastify.authenticate,PlanValidation.dietPlanCreate]}, PlanController.dietPlanCreate)
  fastify.post('/plan/membership/diet-plan/list',{preHandler:[fastify.authenticate,PaginationMiddleware.pagination,PlanValidation.dietPlanList]}, PlanController.dietPlanList)


  fastify.post('/google/auth/fetch',{preHandler:[GoogleValidation.ensureToken]}, GoogleController.fetch)

  fastify.post('/country/list',{preHandler:[PaginationMiddleware.pagination, CountryValidation.list]}, CountryController.list)
  fastify.post('/country/create',{preHandler:[]}, CountryController.create)
}
