const Joi = require("joi");
const {User} = require("../model/user-model");
const {pitLib} = require("../helpers/pitLib");
const {isValidObjectId} = require("mongoose");
const {Plan} = require("../model/PlanModel");
const {PlanMembership} = require("../model/PlanMembership");
const {PlanPurchase} = require("../model/PlanPurchaseModel");
const moment = require("moment");

const create = async (req, res) => {
    let jschema = Joi.object({
        title: Joi.string().required(),
        session: Joi.number().min(0),
        validity: Joi.number().min(0),
        price: Joi.object({
            inr:Joi.number().required(),
            usd:Joi.number().required(),
            eur:Joi.number().required(),
        }).required().min(1),
        description: Joi.string().optional(),
        data: Joi.required()
    })
    let validation=jschema.validate(req.body)
    if (validation.error){
        return res.send(pitLib.sendResponse(0, validation.error.message))
    }
};
const list = async (req, res) => {
    if (req.body._id&&!isValidObjectId(req.body._id)){
        return res.send(pitLib.sendResponse(false,'Invalid ID'))
    }
};
const purchase = async (req, res) => {
    let jschema = Joi.object({
        _id: Joi.required().custom(pitLib.joi.oid)
    })
    let validation=jschema.validate(req.body)
    if (!validation.error){
        p=await Plan.findById(req.body._id)
        if (!p){
            validation.error={message:'Plan not found'}
        }
    }
    if (validation.error){
        return res.send(pitLib.sendResponse(0, validation.error.message))
    }
};
const purchaseList = async (req, res) => {
    if (req.body._id){
        let jschema = Joi.object({
            _id: Joi.required().custom(pitLib.joi.oid)
        })
        let validation=jschema.validate(req.body)
        if (!validation.error){
            p=await PlanPurchase.findById(req.body._id)
            if (!p){
                validation.error={message:'Plan not found'}
            }
        }
        if (validation.error){
            return res.send(pitLib.sendResponse(0, validation.error.message))
        }
    }
    if (req.body.created_by){
        req.body.created_by=pitLib.oid(req.body.created_by)
    }
};
const membersipList = async (req, res) => {
    if (req.body._id){
        let jschema = Joi.object({
            _id: Joi.required().custom(pitLib.joi.oid)
        })
        let validation=jschema.validate(req.body)
        if (!validation.error){
            p=await PlanMembership.findById(req.body._id)
            if (!p){
                validation.error={message:'Not found'}
            }
        }
        if (validation.error){
            return res.send(pitLib.sendResponse(0, validation.error.message))
        }
    }
    if (req.body.created_by){
        req.body.created_by=pitLib.oid(req.body.created_by)
    }
};
const handlePurchase = async (req, res) => {
    // let jschema = Joi.object({
    //     id: Joi.number().required().min(1)
    // })
    // let validation=jschema.validate(req.body)
    // if (validation.error){
    //     return res.send(pitLib.sendResponse(0, validation.error.message))
    // }
};
const dietPlanCreate = async (req, res) => {
    let jschema = Joi.object({
        title: Joi.string().required(),
        content: Joi.string().required(),
        target_date: Joi.string().required(),
        plan_membership: Joi.string().required().custom(pitLib.joi.oid),
    })
    let validation=jschema.validate(req.body)
    if (!validation.error&&!await PlanMembership.findById(pitLib.oid(req.body.plan_membership))){
        validation.error={
            message:"Invalid Membership ID"
        }
    }
    if (validation.error){
        return res.send(pitLib.sendResponse(0, validation.error.message))
    }
    req.body.created_by=req.headers.user_data._id
};
const dietPlanList = async (req, res) => {
    if (req.body._id){
        req.body._id=pitLib.oid(req.body._id)
    }
    if (req.body.plan_membership){
        req.body.plan_membership=pitLib.oid(req.body.plan_membership)
    }
    if (req.body.created_by){
        req.body.created_by=pitLib.oid(req.body.created_by)
    }
    if (req.body.target_date){
        req.body.target_date=pitLib.date.toMongoDate(req.body.target_date)
    }
    // return res.send(req.body)
};
module.exports = {create,list,purchase,handlePurchase,purchaseList,membersipList,dietPlanCreate,dietPlanList}